<?php
class phpDocumentor_HighlightWordParser extends phpDocumentorTWordParser
{
    function setup(&$input, &$parser)
    {
        $this->_parser = &$parser;
        $this->data = &$input;
        $this->_all = $input;
        $this->_sourceline = 0;
        $this->pos = 0;
        $this->linenum = 0;
    }
    
    function printState()
    {
        $linenum = $this->linenum;
        $pos = $this->pos;
        if (!isset($this->_all[$this->linenum][$this->pos]))
        {
            $linenum++;
            $pos = 0;
        }
        $details = '';
        $token = $this->_all[$linenum][$pos];
        if (is_array($token))
        {
            $details = token_name($token[0]);
            $token = htmlspecialchars($token[1]);
        } else $token = htmlspecialchars($token);
        debug('Next Token '.$this->linenum.'-'.$this->pos.':'.$details);
        var_dump($token);
    }
    
    function nextToken()
    {
        $linenum = $this->linenum;
        $pos = $this->pos;
        if (!isset($this->_all[$this->linenum][$this->pos]))
        {
            $linenum++;
            $pos = 0;
        }
        if (!isset($this->_all[$linenum][$pos])) return false;
        return array($linenum, $pos);
    }
    
    function getWord()
    {
        if (!isset($this->_all[$this->linenum][$this->pos]))
        {
            $this->linenum++;
            $this->pos = 0;
            if (!isset($this->_all[$this->linenum])) return false;
            $this->_parser->newLineNum();
            return $this->getWord();
        }
        $word = $this->_all[$this->linenum][$this->pos++];
        return $word;
    }

    function backupPos($last_token, $is_pos = false)
    {
        if ($is_pos)
        {
            $this->linenum = $last_token[0];
            $this->pos = $last_token[1];
            return;
        }
        if ($last_token === false) return;
        //fancy_debug('before',$this->linenum,$this->pos,token_name($this->_all[$this->linenum][$this->pos][0]),htmlentities($this->_all[$this->linenum][$this->pos][1]),$this->_all[$this->linenum]);
        do
        {
            $this->pos--;
            if ($this->pos < 0)
            {
                $this->linenum--;
                $this->pos = count($this->_all[$this->linenum]) - 1;
            }
        } while (!$this->tokenEquals($last_token,$this->_all[$this->linenum][$this->pos]));
        //fancy_debug('after',$this->linenum,$this->pos,token_name($this->_all[$this->linenum][$this->pos][0]),htmlentities($this->_all[$this->linenum][$this->pos][1]));
    }
}

/**
 * @todo document everything
 * @todo make standard tags core tags for highlighting
 */
class phpDocumentor_HighlightParser extends phpDocumentorTParser
{
    var $_output;
    var $_converter;
    var $eventHandlers = array(
                                PARSER_EVENT_ARRAY => 'defaultHandler',
                                PARSER_EVENT_CLASS => 'handleClass',
                                PARSER_EVENT_COMMENT => 'handleComment',
                                PARSER_EVENT_DOCBLOCK_TEMPLATE => 'handleDocBlockTemplate',
                                PARSER_EVENT_END_DOCBLOCK_TEMPLATE => 'handleEndDocBlockTemplate',
                                PARSER_EVENT_LOGICBLOCK => 'handleLogicBlock',
                                PARSER_EVENT_METHOD_LOGICBLOCK => 'handleMethodLogicBlock',
                                PARSER_EVENT_NOEVENTS => 'defaultHandler',
                                PARSER_EVENT_OUTPHP => 'defaultHandler',
                                PARSER_EVENT_CLASS_MEMBER => 'handleClassMember',
                                PARSER_EVENT_DEFINE => 'defaultHandler',
                                PARSER_EVENT_DEFINE_PARAMS => 'defaultHandler',
                                PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS => 'defaultHandler',
                                PARSER_EVENT_INCLUDE_PARAMS_PARENTHESIS => 'defaultHandler',
                                PARSER_EVENT_DOCBLOCK => 'handleDocBlock',
                                PARSER_EVENT_TAGS => 'handleTags',
                                PARSER_EVENT_DESC => 'handleDesc',
                                PARSER_EVENT_DOCKEYWORD => 'handleTag',
                                PARSER_EVENT_DOCKEYWORD_EMAIL => 'handleDockeywordEmail',
                                PARSER_EVENT_EOFQUOTE => 'handleEOFQuote',
                                PARSER_EVENT_FUNCTION => 'handleFunction',
                                PARSER_EVENT_METHOD => 'handleMethod',
                                PARSER_EVENT_FUNCTION_PARAMS => 'handleFunctionParams',
                                PARSER_EVENT_FUNC_GLOBAL => 'handleFuncGlobal',
                                PARSER_EVENT_INLINE_DOCKEYWORD => 'handleInlineDockeyword',
                                PARSER_EVENT_INCLUDE => 'defaultHandler',
                                PARSER_EVENT_INCLUDE_PARAMS => 'defaultHandler',
                                PARSER_EVENT_QUOTE => 'handleQuote',
                                PARSER_EVENT_PHPCODE => 'handlePhpCode',
                                PARSER_EVENT_SINGLEQUOTE => 'handleSingleQuote',
                                PARSER_EVENT_STATIC_VAR => 'defaultHandler',
                                PARSER_EVENT_STATIC_VAR_VALUE => 'defaultHandler',
                                PARSER_EVENT_VAR => 'handleVar',
    );
    
    function newLineNum()
    {
        if ($this->_pf_no_output_yet) return;
        $this->_flush_save();
        $this->_output .= $this->_converter->SourceLine($this->_wp->linenum, $this->_line);
        $this->_line = '';
    }
    
    /**
     * Parse a new file
     *
     * @param    string    $parse_data
     * @param    string    $path
     * @param    int    $base    number of directories to drop off the bottom when creating names using path
     * @staticvar    integer    used for recursion limiting if a handler for an event is not found
     * @return    bool
     * @todo add class knowledge
     */
    function parse (&$parse_data, &$converter, $inlinesourceparse = false)
    {
        static $endrecur = 0;
        $this->_converter = $converter;
        $this->setupStates($inlinesourceparse);

        $this->configWordParser($parse_data);
        // initialize variables so E_ALL error_reporting doesn't complain
        $pevent = 0;
        $word = 0;

        do
        {
            $lpevent = $pevent;
            $pevent = $this->_event_stack->getEvent();
            if ($lpevent != $pevent)
            {
                $this->_last_pevent = $lpevent;
            }

            if ($pevent == PARSER_EVENT_CLASS_MEMBER)
            {
                $this->_wp->setWhitespace(true);
            } else
            {
                $this->_wp->setWhitespace(false);
            }

            if (!is_array($word)) $lw = $word;
            if (is_array($word) && $word[0] != T_WHITESPACE) $lw = $word;
            $dbg_linenum = $this->_wp->linenum;
            $dbg_pos = $this->_wp->getPos();
            $word = $this->_wp->getWord();
            // change tabs to spaces for consistency of output
            if (is_array($word) && $word[0] == T_WHITESPACE) $word[1] = str_replace("\t",'    ',$word[1]);
            if (is_array($word) && $word[0] == T_WHITESPACE  && $pevent != PARSER_EVENT_CLASS_MEMBER)
            {
//                debug("added ".$this->_wp->linenum.'-'.$this->_wp->pos);
                $this->_addoutput($word);
                continue;
            } else $this->_pv_last_word = $lw;
            if ($pevent != PARSER_EVENT_DOCBLOCK) $this->_pv_next_word = $this->_wp->nextToken();
            // in wordparser, have to keep track of lines
//            $this->publishEvent(PHPDOCUMENTOR_EVENT_NEWLINENUM, $this->_wp->linenum);

            if (0)//PHPDOCUMENTOR_DEBUG == true)
            {
                echo "LAST: ";
                if (is_array($this->_pv_last_word))
                {
                    echo token_name($this->_pv_last_word[0]). ' => |'.htmlspecialchars($this->_pv_last_word[1]);
                } else echo "|" . $this->_pv_last_word;
                echo "|\n";
                echo "PEVENT: " . $this->getParserEventName($pevent) . "\n";
                echo "LASTPEVENT: " . $this->getParserEventName($this->_last_pevent) . "\n";
//                echo "LINE: ".$this->_line."\n";
//                echo "OUTPUT: ".$this->_output."\n";
                echo $dbg_linenum.'-'.$dbg_pos . ": ";
                if (is_array($word))
                {
                    echo token_name($word[0]).' => |'.htmlspecialchars($word[1]);
                } else echo '|'.htmlspecialchars($word);
                echo "|\n";
                $this->_wp->printState();
                echo "NEXT TOKEN: ";
                $tok1 = $this->_pv_next_word;
                $tok = $this->_wp->_all[$tok1[0]][$tok1[1]];
                if (is_array($tok))
                {
                    echo token_name($tok[0]). ' => '.$tok1[0].'-'.$tok1[1].'|'.htmlspecialchars($tok[1]);
                } else echo "|" . $tok;
                echo "|\n";
                echo "-------------------\n\n\n";
            }
            if ($this->_pf_get_source)
            {
                if ($pevent == PARSER_EVENT_FUNCTION)
                {
                    $this->_wp->retrievesource($word);
                    $this->_pf_get_source = false;
                    $this->_pf_getting_source = true;
                }
            }
            if (0)//$this->_pf_getting_source && ($pevent == PARSER_EVENT_DOCBLOCK) || ($pevent == PARSER_EVENT_NOEVENTS))
            {
                addError(PDERROR_SOURCE_TAG_FUNCTION_NOT_FOUND);
                // throw away source
                $this->_wp->getSource();
            }
            if (isset($this->eventHandlers[$pevent]))
            {
                $handle = $this->eventHandlers[$pevent];
                $this->$handle($word, $pevent);
            } else
            {
                debug('WARNING: possible error, no handler for event number '.$pevent);
                if ($endrecur++ == 25)
                {
                    die("FATAL ERROR, recursion limit reached");
                }
            }
        } while (!($word === false));
        if (strlen($this->_line)) $this->newLineNum();
        return $this->_converter->PreserveWhiteSpace($this->_output);
    }

    function defaultHandler($word, $pevent)
    {
        $this->_addoutput($word);
        if ($this->checkEventPush($word, $pevent)) return;
        $this->checkEventPop($word, $pevent);
    }
    
    function handleFuncGlobal($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent)) return;
        $this->_globallink($word);
        $this->checkEventPop($word, $pevent);
    }
    
    function handleQuote($word, $pevent)
    {
        if ($this->_pf_inmethod && is_array($word) && $word[0] == T_VARIABLE) $this->_pv_lastvar = $word;
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        if ($this->_pf_quote_active && $this->_pv_last_word == '"' && $this->_last_pevent != PARSER_EVENT_QUOTE)
        {
            $this->_pf_quote_active = false;
            $this->_wp->backupPos($word);
            $this->_event_stack->popEvent();
            return;
        }
        if (!$this->_pf_quote_active && $this->_pv_last_word == '"' && $this->_last_pevent != PARSER_EVENT_QUOTE)
        {
            if (is_array($word) && $word[0] == T_VARIABLE) $this->_pv_lastvar = $word;
            $this->_pf_quote_active = true;
            $this->_addoutput($word);
            $this->checkEventPop($word, $pevent);
            return;
        } elseif (is_array($this->_pv_last_word) && $this->_pv_last_word[0] == T_CONSTANT_ENCAPSED_STRING)
        {
//            $this->_pv_quote_data = $this->_pv_last_word[1];
            $this->_event_stack->popEvent();
            $this->_wp->backupPos($word);
            return;
        }
        if ($this->checkEventPop($word, $pevent))
        {
            $this->_pf_quote_active = false;
        }
        $this->_addoutput($word);
    }
    
    function handleDefine($word, $pevent)
    {
        static $token_save;
        if (!isset($token_save)) $token_save = array();
        $e = $this->checkEventPush( $word, $pevent);
        if ($e && $e != PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS) return;
        
        if(!isset($this->_pv_define_params_data)) $this->_pv_define_params_data = '';
        
        if ($this->checkEventPop($word,$pevent))
        {
            unset($token_save);
            $this->_addoutput($word);
        }
        if ($this->_pf_definename_isset)
        {
            $this->_addoutput($word);
        } else
        {
            if ($word != ",")
            {
                $token_save[] = $word;
                if (is_array($word)) $word = $word[1];
                $this->_pv_define_params_data .= $word;
            } else
            {
                if (substr($this->_pv_define_params_data,0,1) ==
                    substr($this->_pv_define_params_data,strlen($this->_pv_define_params_data) - 1) &&
                    in_array(substr($this->_pv_define_params_data,0,1),array('"',"'")))
                { // remove leading and ending quotation marks if there are only two
                    $a = substr($this->_pv_define_params_data,0,1);
                    $b = substr($this->_pv_define_params_data,1,strlen($this->_pv_define_params_data) - 2);
                    if (strpos($b,$a) === false)
                    {
                        $this->_pv_define_params_data = $b;
                    }
                }
                $this->_pf_definename_isset = true;
                $link = $this->_converter->getLink($this->_pv_define_params_data);
                foreach ($token_save as $token)
                {
                    if (is_object($link))
                    {
                        if (is_array($token)) $token = $token[1];
                        $this->_addoutput($this->_converter->returnSee($link, $token));
                    } else
                    {
                        $this->_addoutput($save, $token);
                    }
                }
                $this->_pv_define_params_data = '';
            }
        }
    }
    
    function handlePhpCode($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent) == PARSER_EVENT_DOCBLOCK)
        {
            $this->_pf_no_output_yet = true;
            $this->_pv_saveline = $this->_wp->linenum + 1;
            return;
        }
        $this->_addoutput($word);
        $this->checkEventPop($word, $pevent);
    }
    
    function handleFunction($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        if ($this->checkEventPop($word, $pevent)) return;
        $this->_link($word);
    }
    
    function handleMethod($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        if ($this->checkEventPop($word, $pevent)) return;
        $this->_methodlink($word);
    }
    
    function handleFunctionParams($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        $this->_addoutput($word);
        $this->checkEventPop($word, $pevent);
    }
    
    function handleLogicBlock($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        $this->_link($word);
        if ($this->checkEventPop($word,$pevent))
        {
            $e = $this->_event_stack->popEvent();
            $this->_event_stack->pushEvent($e);
            if ($e == PARSER_EVENT_FUNCTION)
            {
                $this->_wp->backupPos($word); 
            }
        }
    }
    
    function handleMethodLogicBlock($word, $pevent)
    {
        $this->_pf_inmethod = true;
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        if (is_array($word) && $word[0] == T_DOUBLE_COLON) $this->_pf_colon_colon = true;
        if (!$this->_pf_colon_colon && is_array($word) && $word[0] == T_STRING) $this->_pv_last_string = $word;
        if (is_array($word) && $word[0] == T_VARIABLE) $this->_pv_lastvar = $word;
        $this->_link($word);
        if ($this->checkEventPop($word,$pevent))
        {
            $this->_pf_inmethod = false;
            $e = $this->_event_stack->popEvent();
            $this->_event_stack->pushEvent($e);
            if ($e == PARSER_EVENT_METHOD)
            {
                $this->_wp->backupPos($word); 
            }
        }
    }
    
    function handleClassMember($word, $pevent)
    {
        if (!isset($this->_pv_lastvar))
        {
//            fancy_debug('returned from',$word);
            $this->_event_stack->popEvent();
            return $this->defaultHandler($word, $pevent);
        }
        if (isset($this->_pv_cm_name))
        {
            $this->_pf_obj_op = false;
            $name = $this->_pv_cm_name;
            unset($this->_pv_cm_name);
            unset($this->_pv_lastvar);
//            debug('unset lastvar');
            $this->_event_stack->popEvent();
            if (is_string($word) && $word == '(')
            { // it's a function
                $this->_methodlink($name);
            } else
            { // it's a variable
                $this->_varlink($name, true);
            }
            if ($word[0] == T_OBJECT_OPERATOR)
                $this->_wp->backupPos($word);
            else
                $this->_addoutput($word);
            return;
        }
        if (!$this->_pf_obj_op && is_array($this->_pv_last_word) && $this->_pv_last_word[0] == T_OBJECT_OPERATOR)
        {
            $this->_pf_obj_op = true;
        }
        if (is_array($word) && $word == T_WHITESPACE)
        {
            $this->_addoutput($word);
            return;
        }
        if ($this->_pf_obj_op)
        {
            if (!(is_array($word) && $word[0] == T_STRING))
            {
                unset($this->_pv_lastvar);
//                debug('unset lastvar');
                $this->_event_stack->popEvent();
                $this->_addoutput($word);
                return;
            }
            $this->_pv_cm_name = $word;
        }
    }
    
    function handleComment($word, $pevent)
    {
        if (!is_array($word) || $word[0] != T_COMMENT)
        {
            $this->_event_stack->popEvent();
            $this->_wp->backupPos($word);
            return;
        }
        $this->_addoutput($word);
        $this->checkEventPop($word, $pevent);
    }
    
    function handleDocBlock($word, $pevent)
    {
        if (!($this->_pf_docblock || $this->_pf_docblock_template))
        {
            if (strpos($this->_pv_last_word[1],'/**') !== 0)
            { // not a docblock
                $this->_wp->backupPos($this->_pv_last_word);
                $this->_event_stack->popEvent();
                $this->_event_stack->pushEvent(PARSER_EVENT_COMMENT);
                $this->_pf_no_output_yet = false;
                return;
            } else
            {
                $this->_pf_no_output_yet = true;
                $this->_pv_db_lines = array();
            }
        }
        $last_word = $this->_pv_last_word[1];
        $dtype = '_pv_docblock';
        if ($last_word == '/**#@-*/')
        { // stop using docblock template
            $this->_pf_no_output_yet = false;
            $this->_addDocBlockoutput('closetemplate', $last_word);
            $this->_wp->backupPos($word);
            $this->_event_stack->popEvent();
            return;
        }
        if (!($this->_pf_docblock || $this->_pf_docblock_template))
        {
            $this->_pv_db_lines = array();
            if (strpos($last_word,'/**#@+') === 0)
            { // docblock template definition
                $this->_pf_docblock_template = true;
            } else
            {
                $this->_pf_docblock = true;
            }
            $this->_pv_db_lines[] = $last_word;
            if (strpos($last_word,'*/') !== false)
            {
                return $this->commonDocBlock($word);
            }
            $this->_pv_db_lines[] = $word[1];
            if (strpos($word[1],'*/') !== false)
            {
                $this->commonDocBlock($word);
            }
        } else
        {
            $this->_pv_db_lines[] = $word[1];
        }
        if (($this->_pf_docblock || $this->_pf_docblock_template) && (strpos($word[1],'*/') !== false))
        {
            $this->commonDocBlock($word);
        }
    }
    
    function commonDocBlock($word)
    {
        $this->_event_stack->popEvent();
        $lines = $this->_pv_db_lines;
        $go = count($this->_pv_db_lines);
        for($i=0;$i<$go;$i++)
        {
            if (substr(trim($lines[$i]),0,2) == '*/' || substr(trim($lines[$i]),0,1) != '*' && substr(trim($lines[$i]),0,3) != '/**')
            {
                $lines[$i] = array($lines[$i],false);
            } elseif (substr(trim($lines[$i]),0,3) == '/**')
            {
                $linesi = array();
                $linesi[1] = substr(trim($lines[$i]),3); // remove leading "/**"
                if (empty($linesi[1]))
                $linesi[0] = $lines[$i];
                else
                $linesi[0] = substr($lines[$i],0,strpos($lines[$i],$linesi[1]));
                $lines[$i] = $linesi;
            } else
            {
                $linesi = array();
                $linesi[1] = substr(trim($lines[$i]),1); // remove leading "* "
                if (empty($linesi[1]))
                $linesi[0] = $lines[$i];
                else
                $linesi[0] = substr($lines[$i],0,strpos($lines[$i],$linesi[1]));
                $lines[$i] = $linesi;
            }
        }
        for($i = 0;$i<count($lines);$i++)
        {
            if ($lines[$i][1] === false) continue;
            if (substr(trim($lines[$i][1]),0,1) == '@' && substr(trim($lines[$i][1]),0,2) != '@ ')
            {
                $tagindex = $i;
                $i = count($lines);
            }
        }
        if (isset($tagindex))
        {
            $tags = array_slice($lines,$tagindex);
            $desc = array_slice($lines,0,$tagindex);
        } else
        {
            $tags = array();
            $desc = $lines;
        }
//        var_dump($desc,$tags);
        $this->_pf_no_output_yet = false;
        $save = $this->_wp->linenum;
        $this->_wp->linenum = $this->_pv_saveline;
        $this->handleDesc($desc);
        $this->handleTags($tags);
        $this->_pv_db_lines = array();
        $this->_wp->linenum = $save;
        if (strpos($this->_pv_last_word[1],'*/') !== false)
        {
            $this->_wp->backupPos($this->_pv_next_word,true);
        }
        $this->_pf_docblock = $this->_pf_docblock_template = false;
    }
    
    function handleDesc($desc)
    {
        $dbtype = 'docblock';
        $dbtype .= ($this->_pf_docblock ? '' : 'template');
        foreach($desc as $line)
        {
            $this->getInlineTags($line[0].$line[1]);
            if (strpos($line[0],'*/') === false)
            {
                $this->newLineNum();
                $this->_wp->linenum++;
            }
        }
    }
    
    function handleTags($tags)
    {
        $newtags = array();
        $curtag = array();
        for($i=0;$i < count($tags);$i++)
        {
            $tagsi = trim($tags[$i][1]);
            if (substr($tagsi,0,1) == '@' && substr($tagsi,0,2) != '@ ')
            { // start a new tag
                $tags[$i][1] = array(substr($tags[$i][1],0,strpos($tags[$i][1],$tagsi)),$tagsi);
                if (!empty($curtag))
                {
                    $newtags[] = $curtag;
                    $curtag = array();
                }
                $curtag[] = $tags[$i];
            } else $curtag[] = $tags[$i];
        }
        if (!empty($curtag)) $newtags[] = $curtag;
        foreach($newtags as $tag)
        {
            foreach($tag as $i => $t)
            {
                if ($t[1] === false) continue;
                $tag[$i][1][1] = explode(" ",str_replace("\t",'    ',$t[1][1]));
                if ($i == 0) $x = $tag[$i][1][1];
            }
            $tagname = substr(array_shift($x),1);
            $restoftag = $tag;
            if (isset($this->tagHandlers[$tagname]))
            $handle = $this->tagHandlers[$tagname];
            else
            $handle = $this->tagHandlers['*'];
            $this->$handle($tagname,$restoftag);
        }
    }
    
    function getInlineTags($value, $endinternal = false)
    {
        if ($this->_pf_internal && !$endinternal)
        {
            if ($x = strrpos($value,'}}'))
            {
                // add the rest of internal
                $this->getInlineTags(substr($value,0,$x + 1), true);
                // strip internal from value
                $value = substr($value,strrpos($value,'}}') + 1);
                // turn off internal
                $this->_pf_internal = false;
            }
        }
        $dbtype = 'docblock';
        $dbtype .= ($this->_pf_docblock ? '' : 'template');
        $save = $value;
        $value = explode('{@',$value);
        $newval = array();
        // everything before the first {@ is normal text
        $this->_addDocBlockoutput($dbtype, $value[0]);
        for($i=1;$i<count($value);$i++)
        {
            if (substr($value[$i],0,1) == '}')
            {
                $this->_addDocBlockoutput($dbtype, '{@}'.substr($value[$i],1));
            } else
            {
                $save = $value[$i];
                $value[$i] = str_replace("\t","    ",$value[$i]);
                $value[$i] = explode(" ",$value[$i]);
                $word = array_shift($value[$i]);
                $val = join(' ',$value[$i]);
                if ($word == 'internal')
                {
                    $this->_pf_internal = true;
                    $this->_addDocBlockoutput($dbtype, '{@internal');
                    $value[$i] = substr($save,strlen('internal') + 1);
                    // strip internal and cycle as if it were normal text.
                    $this->_addDocBlockoutput($dbtype, $value[$i]);
                    continue;
                }
                if (in_array(str_replace('}','',$word),$this->allowableInlineTags))
                {
                    if (strpos($word,'}'))
                    {
                        $word = str_replace('}','',$word);
                        $val = '} '.$val;
                    }
                    $val = explode('}',$val);
                    if (count($val) == 1)
                    {
//                         addError(PDERROR_UNTERMINATED_INLINE_TAG,$word,'',$save);
                    }
                    $rest = $val;
                    $val = array_shift($rest);
                    $rest = join(' ',$rest);
                    if (isset($this->inlineTagHandlers[$word]))
                    $handle = $this->inlineTagHandlers[$word];
                    else
                    $handle = $this->inlineTagHandlers['*'];
                    $this->$handle($word,$val);
                    $this->_addDocBlockoutput($dbtype, $rest);
                } else
                {
                    $val = $word.' '.$val;
                    $this->_addDocBlockoutput($dbtype, '{@'.$val);
                }
            }
        }
    }
    
    function handleDefaultInlineTag($name, $value)
    {
        $this->_addDocBlockoutput('inlinetag','{@'.$name.' '.$value.'}');
    }

    function defaultTagHandler($name, $value, $coretag = false)
    {
        $dbtype = 'docblock';
        $dbtype .= ($this->_pf_docblock ? '' : 'template');
        foreach($value as $line)
        {
            $this->_addDocBlockoutput($dbtype, $line[0]);
            if ($line[1] === false)
            {
                if (trim($line[0]) != '*/')
                {
                    $this->newLineNum();
                    $this->_wp->linenum++;
                }
                continue;
            }
            $this->_addDocBlockoutput($dbtype, $line[1][0]);
            $stored = '';
            foreach($line[1][1] as $i => $tpart)
            {
                if ($tpart == '@'.$name && $i == 0)
                {
                    $tagname = 'tag';
                    if ($coretag) $tagname = 'coretag';
                    $this->_addDocBlockoutput($tagname,'@'.$name);
                    continue;
                }
                $stored .= ' '.$tpart;
            }
            $this->getInlineTags($stored);
            if (strpos($stored,'*/') === false)
            {
                $this->newLineNum();
                $this->_wp->linenum++;
            }
        }
    }
    
    function authorTagHandler($name, $value)
    {
        return $this->defaultTagHandler($name, $value);
    }
    
    function packageTagHandler($name, $value)
    {
        return $this->defaultTagHandler($name, $value, true);
    }
    
    function globalTagHandler($name, $value)
    {
        $dbtype = 'docblock';
        $dbtype .= ($this->_pf_docblock ? '' : 'template');
        foreach($value as $num => $line)
        {
            $this->_addDocBlockoutput($dbtype, $line[0]);
            if ($line[1] === false)
            {
                if (trim($line[0]) != '*/')
                {
                    $this->newLineNum();
                    $this->_wp->linenum++;
                }
                continue;
            }
            $this->_addDocBlockoutput($dbtype, $line[1][0]);
            $stored = '';
            $typeloc = 1;
            $varloc = 2;
            foreach($line[1][1] as $i => $tpart)
            {
                if ($tpart == '@'.$name && $i == 0)
                {
                    $this->_addDocBlockoutput('coretag','@'.$name);
                    continue;
                }
                if (empty($tpart))
                {
                    $typeloc++;
                    $varloc++;
                }
                if ($num == 0 && $i == $typeloc)
                {
                    $this->_addDocBlockoutput('tagphptype',' '.$tpart);
                    continue;
                }
                if ($num == 0 && $i == $varloc)
                {
                    if (substr($tpart,0,1) == '$')
                    {
                        $this->_addDocBlockoutput('tagvarname',' '.$tpart);
                        continue;
                    }
                }
                $stored .= ' '.$tpart;
            }
            $this->getInlineTags($stored);
            if (strpos($stored,'*/') === false)
            {
                $this->newLineNum();
                $this->_wp->linenum++;
            }
        }
    }
    
    function staticvarTagHandler($name, $value)
    {
        return $this->paramTagHandler($name, $value);
    }
    
    function paramTagHandler($name, $value)
    {
        $dbtype = 'docblock';
        $dbtype .= ($this->_pf_docblock ? '' : 'template');
        foreach($value as $num => $line)
        {
            $this->_addDocBlockoutput($dbtype, $line[0]);
            if ($line[1] === false)
            {
                if (trim($line[0]) != '*/')
                {
                    $this->newLineNum();
                    $this->_wp->linenum++;
                }
                continue;
            }
            $this->_addDocBlockoutput($dbtype, $line[1][0]);
            $stored = '';
            $typeloc = 1;
            $varloc = 2;
            foreach($line[1][1] as $i => $tpart)
            {
                if ($tpart == '@'.$name && $i == 0)
                {
                    $this->_addDocBlockoutput('coretag','@'.$name);
                    continue;
                }
                if (empty($tpart))
                {
                    $typeloc++;
                    $varloc++;
                }
                if ($num == 0 && $i == $typeloc)
                {
                    $this->_addDocBlockoutput('tagphptype',' '.$tpart);
                    continue;
                }
                if ($num == 0 && $i == $varloc)
                {
                    if (substr($tpart,0,1) == '$')
                    {
                        $this->_addDocBlockoutput('tagvarname',' '.$tpart);
                        continue;
                    }
                }
                $stored .= ' '.$tpart;
            }
            $this->getInlineTags($stored);
            if (strpos($stored,'*/') === false)
            {
                $this->newLineNum();
                $this->_wp->linenum++;
            }
        }
    }
    
    function returnTagHandler($name, $value)
    {
        $dbtype = 'docblock';
        $dbtype .= ($this->_pf_docblock ? '' : 'template');
        foreach($value as $num => $line)
        {
            $this->_addDocBlockoutput($dbtype, $line[0]);
            if ($line[1] === false)
            {
                if (trim($line[0]) != '*/')
                {
                    $this->newLineNum();
                    $this->_wp->linenum++;
                }
                continue;
            }
            $this->_addDocBlockoutput($dbtype, $line[1][0]);
            $stored = '';
            $typeloc = 1;
            foreach($line[1][1] as $i => $tpart)
            {
                if ($tpart == '@'.$name && $i == 0)
                {
                    $this->_addDocBlockoutput('coretag','@'.$name);
                    continue;
                }
                if (empty($tpart))
                {
                    $typeloc++;
                }
                if ($num == 0 && $i == $typeloc)
                {
                    $this->_addDocBlockoutput('tagphptype',' '.$tpart);
                    continue;
                }
                $stored .= ' '.$tpart;
            }
            $this->getInlineTags($stored);
            if (strpos($stored,'*/') === false)
            {
                $this->newLineNum();
                $this->_wp->linenum++;
            }
        }
    }
    
    function varTagHandler($name, $value)
    {
        return $this->paramTagHandler($name, $value);
    }

    function handleClass($word, $pevent)
    {
        $this->_pf_in_class = true;
        $a = $this->checkEventPush( $word, $pevent);

        if (!isset($this->_pv_class) && is_array($word) && $word[0] == T_STRING)
        {
            $this->_pv_class = $this->_converter->class = $word[1];
            $this->_classlink($word);
            return;
        }

        if ($this->_pf_extends_found && is_array($word) && $word[0] == T_STRING)
        {
            $this->_classlink($word);
            return;
        }
        if (is_array($word) && $word[0] == T_EXTENDS) $this->_pf_extends_found = true;
        if ($a == PARSER_EVENT_DOCBLOCK)
        {
            $this->_pf_no_output_yet = true;
            $this->_pv_saveline = $this->_wp->linenum + 1;
            return;
        }
        $this->_addoutput($word);
        if ($this->checkEventPop($word,$pevent))
        {
            $this->_pf_in_class = false;
            unset($this->_pv_class);
        }
    }
    
    function handleVar($word, $pevent)
    {
        if ($this->checkEventPush($word, $pevent))
        {
            $this->_addoutput($word);
            return;
        }
        if (is_array($word) && $word[0] == T_VARIABLE)
        {
            return $this->_varlink($word);
        }
        $this->_addoutput($word);
        $this->checkEventPop($word, $pevent);
    }
    
    function _link($word)
    {
        if (is_array($word) && $word[0] == T_STRING)
        {
            if ($this->_pf_colon_colon)
            {
                $this->_pf_colon_colon = false;
                $combo = $this->_pv_last_string[1].'::'.$word[1].'()';
//                debug('testing '.$combo);
                $link = $this->_converter->getLink($combo);
                if (is_object($link))
                {
                    $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                    return;
                }
                $this->_addoutput($word);
                return;
            }
            $link = $this->_converter->getLink($word[1].'()');
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            } elseif (is_string($link) && strpos($link,'ttp://'))
            {
                $this->_addoutput($this->_converter->returnLink($link, $word[1]), true);
                return;
            } else
            {
                $link = $this->_converter->getLink($word[1]);
                if (is_object($link)) $word[1] = $this->_converter->returnSee($link, $word[1]);
                $this->_addoutput($word, true);
                return;
            }
        }
        $this->_addoutput($word);
    }
    
    function _globallink($word)
    {
        if (!is_array($word)) return $word;
        if ($word[0] != T_VARIABLE) return $word;
        if (is_array($word) && $word[0] == T_VARIABLE)
        {
            $link = $this->_converter->getLink('global '.$word[1]);
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            }
        }
        $this->_addoutput($word);
    }
    
    function _classlink($word)
    {
//            debug("checking class ".$word[1]);
        if (is_array($word) && $word[0] == T_STRING)
        {
            $link = $this->_converter->getLink($word[1]);
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            }
        }
        $this->_addoutput($word);
    }
    
    function _methodlink($word)
    {
        if (is_array($word) && $word[0] == T_STRING)
        {
//            debug("checking method ".$this->_pv_class.'::'.$word[1].'()');
            $link = $this->_converter->getLink($this->_pv_class.'::'.$word[1].'()');
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            }
//            debug("checking method ".$word[1].'()');
            $link = $this->_converter->getLink($word[1].'()');
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            }
        }
        $this->_addoutput($word);
    }
    
    function _varlink($word, $justastring=false)
    {
        if ($justastring)
        {
            $word[0] = T_VARIABLE;
        }
        if (is_array($word) && $word[0] == T_VARIABLE)
        {
            $x = ($justastring ? '$' : '');
//            debug("checking var ".$this->_pv_class.'::'.$x.$word[1]);
            $link = $this->_converter->getLink($this->_pv_class.'::'.$x.$word[1]);
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            }
//            debug("checking var ".$x.$word[1]);
            $link = $this->_converter->getLink($x.$word[1]);
            if (is_object($link))
            {
                $this->_addoutput($this->_converter->returnSee($link, $word[1]), true);
                return;
            }
        }
        $this->_addoutput($word);
    }
    
    function _addoutput($word, $preformatted = false)
    {
        if ($this->_pf_no_output_yet) return;
        if ($this->_pf_quote_active)
        {
            if (is_array($word)) $this->_save .= $this->_converter->highlightSource($word[0], $word[1]);
            else
            $this->_save .= $this->_converter->highlightSource(false, $word, true);
        } else
        {
            $this->_flush_save();
            if (is_array($word)) $this->_line .= $this->_converter->highlightSource($word[0],$word[1], $preformatted);
            else
            $this->_line .= $this->_converter->highlightSource(false, $word, $preformatted);
        }
    }
    
    function _addDocBlockoutput($dbtype, $word, $preformatted = false)
    {
        if ($this->_pf_internal)
        {
            $this->_line .= $this->_converter->highlightDocBlockSource('internal', $word, $preformatted);
        } else
        $this->_line .= $this->_converter->highlightDocBlockSource($dbtype, $word, $preformatted);
    }
    
    function _flush_save()
    {
        if (!empty($this->_save))
        {
            $this->_line .= $this->_converter->highlightSource(T_CONSTANT_ENCAPSED_STRING, $this->_save, true);
            $this->_save = '';
        }
    }
    
    function configWordParser(&$data)
    {
        $this->_wp->setup($data, $this);
        $this->_wp->setWhitespace(true);
    }

    function setupStates($inlinesourceparse)
    {
        $this->_output = '';
        $this->_line = '';
        unset($this->_wp);
        $this->_wp = new phpDocumentor_HighlightWordParser;
        $this->_pv_class = null;
        $this->_pv_cur_class = null;
        $this->_pv_define = null;
        $this->_pv_define_name = null;
        $this->_pv_define_value = null;
        $this->_pv_define_params_data = null;
        $this->_pv_dtype = null;
        $this->_pv_docblock = null;
        $this->_pv_dtemplate = null;
        $this->_pv_func = null;
        $this->_pv_global_name = null;
        $this->_pv_global_val = null;
        $this->_pv_globals = null;
        $this->_pv_global_count = null;
        $this->_pv_include_params_data = null;
        $this->_pv_include_name = null;
        $this->_pv_include_value = null;
        $this->_pv_linenum = null;
        $this->_pv_periodline = null;
        $this->_pv_paren_count = 0;
        $this->_pv_statics = null;
        $this->_pv_static_count = null;
        $this->_pv_static_val = null;
        $this->_pv_quote_data = null;
        $this->_pv_function_data = null;
        $this->_pv_var = null;
        $this->_pv_varname = null;
        $this->_pf_definename_isset = false;
        $this->_pf_extends_found = false;
        $this->_pf_includename_isset = false;
        $this->_pf_get_source = false;
        $this->_pf_getting_source = false;
        $this->_pf_in_class = false;
        $this->_pf_in_define = false;
        $this->_pf_in_global = false;
        $this->_pf_in_include = false;
        $this->_pf_in_var = false;
        $this->_pf_funcparam_val = false;
        $this->_pf_quote_active = false;
        $this->_pf_reset_quote_data = true;
        $this->_pf_useperiod = false;
        $this->_pf_var_equals = false;
        $this->_pf_obj_op = false;
        $this->_pf_docblock = false;
        $this->_pf_docblock_template = false;
        $this->_pf_colon_colon = false;
        $this->_pv_last_string = false;
        $this->_pf_inmethod = false;
        $this->_pf_no_output_yet = false;
        $this->_pv_saveline = 0;
        $this->_pv_next_word = false;
        $this->_save = '';
        $this->_event_stack = new EventStack;
        if ($inlinesourceparse)
        {
            $this->_event_stack->pushEvent(PARSER_EVENT_PHPCODE);
        }
    }

    function phpDocumentor_HighlightParser()
    {
        $this->allowableTags = $GLOBALS['_phpDocumentor_tags_allowed'];
        $this->allowableInlineTags = $GLOBALS['_phpDocumentor_inline_doc_tags_allowed'];
        $this->tagHandlers['author'] = 'authorTagHandler';
        $this->tagHandlers['subpackage'] = 'packageTagHandler';
        $this->inlineTagHandlers = array('*' => 'handleDefaultInlineTag');
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_NOEVENTS] = 
            array(
                T_OPEN_TAG => PARSER_EVENT_PHPCODE,
            );

/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_PHPCODE] = 
            array(
                T_FUNCTION     => PARSER_EVENT_FUNCTION,
                T_CLASS     => PARSER_EVENT_CLASS,
                T_INCLUDE_ONCE => PARSER_EVENT_INCLUDE,
                T_INCLUDE => PARSER_EVENT_INCLUDE,
                T_REQUIRE    => PARSER_EVENT_INCLUDE,
                T_REQUIRE_ONCE    => PARSER_EVENT_INCLUDE,
                T_COMMENT   => PARSER_EVENT_DOCBLOCK,
            );
        $this->wordpushEvent[PARSER_EVENT_PHPCODE] =
            array(
                "define"     => PARSER_EVENT_DEFINE,
            );
/**************************************************************/

        $this->wordpushEvent[PARSER_EVENT_FUNCTION] =
            array(
                '{'     => PARSER_EVENT_LOGICBLOCK,
                '('     => PARSER_EVENT_FUNCTION_PARAMS,
            );
        $this->tokenpushEvent[PARSER_EVENT_FUNCTION] =
            array(
                T_COMMENT   => PARSER_EVENT_COMMENT,
                T_ML_COMMENT => PARSER_EVENT_COMMENT,
            );

        $this->wordpopEvent[PARSER_EVENT_FUNCTION] = array("}");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_FUNCTION_PARAMS] =
            array(
                T_CONSTANT_ENCAPSED_STRING => PARSER_EVENT_QUOTE,
                T_ARRAY => PARSER_EVENT_ARRAY,
                T_COMMENT => PARSER_EVENT_COMMENT,
                T_ML_COMMENT => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_FUNCTION_PARAMS] =
            array(
                '"' => PARSER_EVENT_QUOTE,
                "'" => PARSER_EVENT_QUOTE,
            );
        $this->wordpopEvent[PARSER_EVENT_FUNCTION_PARAMS] = array(")");
/**************************************************************/

        $this->wordpushEvent[PARSER_EVENT_LOGICBLOCK] = 
            array(
                "{"    => PARSER_EVENT_LOGICBLOCK,
                '"'    => PARSER_EVENT_QUOTE,
            );
        $this->tokenpushEvent[PARSER_EVENT_LOGICBLOCK] =
            array(
                T_GLOBAL    => PARSER_EVENT_FUNC_GLOBAL,
                T_STATIC    => PARSER_EVENT_STATIC_VAR,
                T_CURLY_OPEN    => PARSER_EVENT_LOGICBLOCK,
                T_DOLLAR_OPEN_CURLY_BRACES => PARSER_EVENT_LOGICBLOCK,
            );

        $this->wordpopEvent[PARSER_EVENT_LOGICBLOCK] = array("}");
        $this->tokenpopEvent[PARSER_EVENT_LOGICBLOCK] = array(T_CURLY_OPEN);

/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_ARRAY] = 
            array(
                T_COMMENT  => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
            );
        $this->wordpopEvent[PARSER_EVENT_ARRAY] = array(")");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_FUNC_GLOBAL] =
            array(
                T_COMMENT   => PARSER_EVENT_COMMENT,
                T_ML_COMMENT    => PARSER_EVENT_COMMENT,
            );

        $this->wordpopEvent[PARSER_EVENT_FUNC_GLOBAL] = array(";");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_STATIC_VAR] =
            array(
                T_CONSTANT_ENCAPSED_STRING  => PARSER_EVENT_QUOTE,
                T_COMMENT   => PARSER_EVENT_COMMENT,
                T_ML_COMMENT    => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_STATIC_VAR] =
            array(
                "="        => PARSER_EVENT_STATIC_VAR_VALUE,
            );
        $this->wordpopEvent[PARSER_EVENT_STATIC_VAR] = array(";");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_STATIC_VAR_VALUE] = 
            array(
                T_CONSTANT_ENCAPSED_STRING  => PARSER_EVENT_QUOTE,
                T_COMMENT   => PARSER_EVENT_COMMENT,
                T_ML_COMMENT    => PARSER_EVENT_COMMENT,
                T_ARRAY     => PARSER_EVENT_ARRAY,
            );
        $this->wordpushEvent[PARSER_EVENT_STATIC_VAR_VALUE] =
            array(
                '"' => PARSER_EVENT_QUOTE,
                "'" => PARSER_EVENT_QUOTE,
            );
        $this->wordpopEvent[PARSER_EVENT_STATIC_VAR_VALUE] = array(";",",");
/**************************************************************/
        $this->tokenpushEvent[PARSER_EVENT_QUOTE] = 
            array(
                T_OBJECT_OPERATOR => PARSER_EVENT_CLASS_MEMBER,
            );

        $this->wordpopEvent[PARSER_EVENT_QUOTE] = array('"');
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_DEFINE] = 
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
                T_CONSTANT_ENCAPSED_STRING        => PARSER_EVENT_QUOTE,
            );
        $this->wordpushEvent[PARSER_EVENT_DEFINE] = 
            array(
                "("     => PARSER_EVENT_DEFINE_PARAMS,
            );
        $this->wordpopEvent[PARSER_EVENT_DEFINE] = array(";");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_DEFINE_PARAMS] = 
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_DEFINE_PARAMS] = 
            array(
                "("    =>    PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS,
                '"' => PARSER_EVENT_QUOTE,
                "'" => PARSER_EVENT_QUOTE,
            );
        $this->wordpopEvent[PARSER_EVENT_DEFINE_PARAMS] = array(")");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS] =
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS] =
            array(
                "("    =>    PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS,
                '"' => PARSER_EVENT_QUOTE,
                "'" => PARSER_EVENT_QUOTE,
            );
        $this->wordpopEvent[PARSER_EVENT_DEFINE_PARAMS_PARENTHESIS] = array(")");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_VAR] = 
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
                T_ARRAY     => PARSER_EVENT_ARRAY,
            );
        $this->wordpopEvent[PARSER_EVENT_VAR] = array(";");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_CLASS] = 
            array(
                T_FUNCTION     => PARSER_EVENT_METHOD,
                T_VAR         => PARSER_EVENT_VAR,
                T_COMMENT         => PARSER_EVENT_DOCBLOCK,
                T_ML_COMMENT         => PARSER_EVENT_DOCBLOCK,
                T_CLOSE_TAG        => PARSER_EVENT_OUTPHP,
            );
        $this->wordpopEvent[PARSER_EVENT_CLASS] = array("}");

/**************************************************************/

        $this->wordpushEvent[PARSER_EVENT_METHOD] =
            array(
                '{'     => PARSER_EVENT_METHOD_LOGICBLOCK,
                '('     => PARSER_EVENT_FUNCTION_PARAMS,
            );
        $this->tokenpushEvent[PARSER_EVENT_METHOD] =
            array(
                T_COMMENT   => PARSER_EVENT_COMMENT,
                T_ML_COMMENT => PARSER_EVENT_COMMENT,
            );

        $this->wordpopEvent[PARSER_EVENT_METHOD] = array("}");
/**************************************************************/

        $this->wordpushEvent[PARSER_EVENT_METHOD_LOGICBLOCK] = 
            array(
                "{"    => PARSER_EVENT_METHOD_LOGICBLOCK,
                '"'    => PARSER_EVENT_QUOTE,
            );
        $this->tokenpushEvent[PARSER_EVENT_METHOD_LOGICBLOCK] =
            array(
                T_OBJECT_OPERATOR => PARSER_EVENT_CLASS_MEMBER,
                T_GLOBAL    => PARSER_EVENT_FUNC_GLOBAL,
                T_STATIC    => PARSER_EVENT_STATIC_VAR,
                T_CURLY_OPEN    => PARSER_EVENT_LOGICBLOCK,
                T_DOLLAR_OPEN_CURLY_BRACES => PARSER_EVENT_LOGICBLOCK,
            );

        $this->wordpopEvent[PARSER_EVENT_METHOD_LOGICBLOCK] = array("}");
        $this->tokenpopEvent[PARSER_EVENT_METHOD_LOGICBLOCK] = array(T_CURLY_OPEN);
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_INCLUDE] = 
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_INCLUDE] = 
            array(
                "("     => PARSER_EVENT_INCLUDE_PARAMS,
            );
        $this->wordpopEvent[PARSER_EVENT_INCLUDE] = array(";");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_INCLUDE_PARAMS] = 
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_INCLUDE_PARAMS] = 
            array(
                "("    =>    PARSER_EVENT_INCLUDE_PARAMS_PARENTHESIS,
            );
        $this->wordpopEvent[PARSER_EVENT_INCLUDE_PARAMS] = array(")");
/**************************************************************/

        $this->tokenpushEvent[PARSER_EVENT_INCLUDE_PARAMS_PARENTHESIS] =
            array(
                T_COMMENT     => PARSER_EVENT_COMMENT,
                T_ML_COMMENT     => PARSER_EVENT_COMMENT,
            );
        $this->wordpushEvent[PARSER_EVENT_INCLUDE_PARAMS_PARENTHESIS] =
            array(
                "("    =>    PARSER_EVENT_INCLUDE_PARAMS_PARENTHESIS,
            );
        $this->wordpopEvent[PARSER_EVENT_INCLUDE_PARAMS_PARENTHESIS] = array(")");
    }
}
?>