<?php
/**
 * All abstract representations of inline tags are in this file
 * @package phpDocumentor
 * @subpackage InlineTags
 * @since separate file since version 1.2
 */
/**
 * Use this element to represent an {@inline tag} like link (see the phpDocumentor spec for more information about inline tags)
 * @see parserStringWithInlineTags
 * @package phpDocumentor
 * @subpackage InlineTags
 * @author Greg Beaver <cellog@users.sourceforge.net>
 * @since 1.0rc1
 * @version $Id: InlineTags.inc,v 1.10 2003/02/02 20:43:01 CelloG Exp $
 */
class parserInlineTag extends parserBase
{
    /**
     * Type is used by many functions to skip the hassle of if get_class($blah) == 'parserBlah'
     * always 'inlinetag'
     * @var string
     */
    var $type = 'inlinetag';
    /**
     * the name of the inline tag (like link)
     * @var string
     */
    var $inlinetype = '';
    
    /**
     * @param string $type tag type (example: link)
     * @param string $value tag value (example: what to link to)
     */
    function parserInlineTag($type,$value)
    {
        $this->inlinetype = $type;
        $this->value = trim($value);
    }
    
    /**
     * @return integer length of the tag
     */
    function Strlen()
    {
        return strlen($this->value);
    }
    
    /**
     * @return string always '', used by {@link Parser::handleDocBlock()} to calculate the short description of a DocBlock
     * @see parserStringWithInlineTags::getString()
     * @see parserStringWithInlineTags::trimmedStrlen()
     */
    function getString()
    {
        return '';
    }
}

/**
 * represents inline links
 * @package phpDocumentor
 * @subpackage InlineTags
 * @author Greg Beaver <cellog@users.sourceforge.net>
 * @since 1.0rc1
 * @version $Id: InlineTags.inc,v 1.10 2003/02/02 20:43:01 CelloG Exp $
 */
class parserLinkInlineTag extends parserInlineTag
{
    /**
     * text to display in the link, can be different from the link for standard links like websites
     * @var string
     */
    var $linktext = '';
    
    /**
     * @param string $link stored in $value, see {@link parserBase::$value}
     * @param string $text see {@link $linktext}
     */
    function parserLinkInlineTag($link,$text)
    {
        parserInlineTag::parserInlineTag('link',$link);
        $this->linktext = trim($text);
    }
    
    /**
     * @param Converter converter used to change the abstract link into text for display
     * @return mixed returns the converted link or false if not converted successfully
     */
    function Convert(&$c)
    {
        if (strpos($this->value,'://') || (strpos($this->value,'mailto:') === 0))
        {
            return $c->returnLink($this->value,htmlspecialchars($this->linktext));
        } else
        {
            $descrip = false;
            if (strpos($this->value,',') === false)
            {
                if (strpos(trim($this->value),' '))
                {
                    $v = split(' ',trim($this->value));
                    $value = $c->getLink($v[0]);
                    array_shift($v);
                    $descrip = join($v,' ');
                } else
                $value = $c->getLink($this->value);
            } else
            {
                $vals = split(',',$this->value);
                $descrip = array();
                foreach($vals as $val)
                {
                    $val = trim($val);
                    if (strpos($val,' '))
                    {
                        $v = split(' ',$val);
                        $value[] = $c->getLink($v[0]);
                        array_shift($v);
                        $descrip[] = join($v,' ');
                    } else
                    {
                        $value[] = $c->getLink($val);
                        $descrip[] = false;
                    }
                }
            }
            if (is_string($value))
            {
                // feature 564991
                if (strpos($value,'://'))
                {
                    // php function
                    return $c->returnLink($value,str_replace('PHP_MANUAL#','',$this->value));
                }
                return $value;
            }
            if (!$descrip) $descrip = $c->type_adjust($this->linktext);
            if (is_object($value)) return $c->returnSee($value,$descrip);
            // getLink parsed a comma-delimited list of linked thingies, add the commas back in
            if (is_array($value))
            {
                $a = '';
                foreach($value as $i => $bub)
                {
                    if (!empty($a)) $a .= ', ';
                    if (is_string($value[$i]))
                    {
                        // feature 564991
                        if (strpos($value[$i],'://'))
                        {
                            // php function
                            $a .= $c->returnLink($value[$i],str_replace('PHP_MANUAL#','',$vals[$i]));
                        } else
                        $a .= $value[$i];
                    }
                    if (is_object($value[$i])) $a .= $c->returnSee($value[$i],$descrip[$i]);
                }
                return $a;
            }
            return false;
        }
    }
}

/**
 * @package phpDocumentor
 * @subpackage InlineTags
 */
class parserTutorialInlineTag extends parserLinkInlineTag
{
    /**
     * @param string $link stored in $value, see {@link parserBase::$value}
     * @param string $text see {@link $linktext}
     */
    function parserTutorialInlineTag($link,$text)
    {
        parserInlineTag::parserInlineTag('tutorial',$link);
        $this->linktext = trim($text);
    }

    /**
     * @param Converter converter used to change the abstract link into text for display
     * @return mixed returns the converted link or false if not converted successfully
     */
    function Convert(&$c)
    {
        $descrip = false;
        if (strpos($this->value,',') === false)
        {
            if (strpos(trim($this->value),' '))
            {
                $v = split(' ',trim($this->value));
                $value = $c->getTutorialLink($v[0]);
                array_shift($v);
                $descrip = join($v,' ');
            } else $value = $c->getTutorialLink($this->value);
        } else
        {
            $vals = split(',',$this->value);
            $descrip = array();
            foreach($vals as $val)
            {
                $val = trim($val);
                if (strpos($val,' '))
                {
                    $v = split(' ',$val);
                    $value[] = $c->getTutorialLink($v[0]);
                    array_shift($v);
                    $descrip[] = join($v,' ');
                } else
                {
                    $value[] = $c->getTutorialLink($val);
                    $descrip[] = false;
                }
            }
        }
        if (is_string($value))
        {
            return $value;
        }
        if (is_object($value)) return $c->returnSee($value,$descrip);
        // getLink parsed a comma-delimited list of linked thingies, add the commas back in
        if (is_array($value))
        {
            $a = '';
            foreach($value as $i => $bub)
            {
                if (!empty($a)) $a .= ', ';
                if (is_string($value[$i]))
                {
                    $a .= $value[$i];
                }
                if (is_object($value[$i])) $a .= $c->returnSee($value[$i],$descrip[$i]);
            }
            return $a;
        }
        return false;
    }
}

/**
 * represents inline source tag
 * @package phpDocumentor
 * @subpackage InlineTags
 */
class parserSourceInlineTag extends parserInlineTag
{
    var $inlinetype = 'source';
    var $value;
    var $start = 0;
    var $end = '*';
    var $source = false;
    /**#@+ @access private */
    /** @var boolean */
    var $_inmethod;
    /** @var string */
    var $_class;
    /**#@-*/
    function parserSourceInlineTag($value)
    {
        parserInlineTag::parserInlineTag('source','');
        preg_match('/^([0-9]*)\W([0-9]*)$/',trim($value), $match);
        if (!count($match))
        {
            preg_match('/^([0-9]*)$/',trim($value),$match);
            if (count($match))
            {
                $this->start = (int) $match[1];
            }
        } else
        {
            $this->start = (int) $match[1];
            $this->end = (int) $match[2];
        }
    }
    
    /**
     * only used to determine blank lines.  { @source } will not be blank, probably
     */
    function Strlen()
    {
        return 1;
    }
    
    function setSource($source, $is_method = false, $class = '')
    {
        if (is_array($source))
        {
            $this->_inmethod = $is_method;
            $this->_class = $class;
            $this->source = $source;
        } else
        {
            $source = strstr($source,'function');
            $pos = strrpos($source,'}');
            $this->source = substr($source,0,$pos + 1);
        }
    }
    
    function Convert(&$c)
    {
        if (is_string($this->source)) return $this->stringConvert($c);
        return $this->arrayConvert($c);
    }
    
    /**
     * @todo add class knowledge
     */
    function arrayConvert(&$c)
    {
        if ($this->end != '*')
        {
            $source = array_slice($this->source,$this->start - 1,$this->end - $this->start + 1);
        } else
        {
            $source = $this->source; // array_slice($this->source,$this->start);
        }
        $parser = new phpDocumentor_HighlightParser;
        $return = '';
        return $parser->parse($source, $c, true);
    }
    
    function stringConvert(&$c)
    {
        $source = highlight_string('<?php '.$this->source.' ?>', true);
        $source = '<code>'.substr($source,strlen('<code><font color="#000000">
<font color="#0000CC">&lt;?php&nbsp;</font>') - 1);
        $source = str_replace('}&nbsp;</font><font color="#0000CC">?&gt;</font>','}</font></code>',$source);
        if ($this->start || ($this->end != '*'))
        {
            $source = explode('<br />',$source);
            $start = $this->start;
            if ($this->end != '*')
            {
                $source = array_slice($source,$start - 1,$this->end - $start + 1);
            } else
            {
                $source = array_slice($source,$start);
            }
            $source = implode($source,'<br />');
            if ($start > 0) $source = "<code>$source";
            if ($this->end != '*') $source = "$source</code>";
        }
        $source = $c->unmangle($source,$this->source);
        return $source;
    }
}

/**
 * represents the inheritdoc inline tag, used by classes/methods/vars to inherit documentation from the parent class if possible
 * @package phpDocumentor
 * @subpackage InlineTags
 */
class parserInheritdocInlineTag extends parserInlineTag
{
    var $inlinetype = 'inheritdoc';
    
    function parserInheritdocInlineTag()
    {
    }
}

/**
 * @package phpDocumentor
 * @subpackage InlineTags
 */
class parserIdInlineTag extends parserInlineTag
{
    var $inlinetype = 'id';
    var $package = 'default';
    var $subpackage = '';
    var $tutorial;
    var $id;
    
    function parserIdInlineTag($package,$subpackage,$tutorial,$id = false)
    {
        $this->package = $package;
        $this->subpackage = $subpackage;
        $this->tutorial = $tutorial;
        $this->id = $id;
    }
    
    function Convert(&$c)
    {
        if (!$this->id) return '';
        return $c->getTutorialId($this->package,$this->subpackage,$this->tutorial,$this->id);
    }
}

/**
 * @package phpDocumentor
 * @subpackage InlineTags
 */
class parserTocInlineTag extends parserInlineTag
{
    var $inlinetype = 'toc';
    /**
     * @var array format: array(array('tagname' => section, 'link' => returnsee link, 'id' => anchor name, 'title' => from title tag),...)
     */
    var $_toc = false;
    var $_path = false;

    function parserTocInlineTag()
    {
        parent::parserInlineTag('toc','');
    }
    
    /**
     * @param array format: array(array('tag' => {@link parserXMLDocBookTag},'id' => {@link parserIdInlineTag}, 'title' => {@link parserXMLDocBookTag title}),...)
     */
    function setTOC($toc)
    {
        $this->toc = $toc;
    }
    
    function setPath($path)
    {
        $this->_path = $path;
    }
    
    /**
     * @uses Converter::formatTutorialTOC() passes an array of format:
     *
     * <pre>
     * array(
     *    'tagname' => string name of tag,
     *    'link' => {@link tutorialLink} to the tutorial,
     *    'id' => converter specific tutorial ID from {@link Converter::getTutorialId()}
     *    'title' => title of the tutorial)
     * </pre>
     *
     * and returns the results as the table of contents
     */
    function Convert(&$c)
    {
        $newtoc = array();
        foreach($this->toc as $i => $toc)
        {
            if (isset($toc['title']))
            $toc['tag']->setTitle($toc['title']);
            else
            $toc['tag']->setTitle(new parserStringWithInlineTags);
            $newtoc[$i]['tagname'] = $toc['tag']->name;
            $l = new tutorialLink;
            if (!isset($toc['title'])) $title = 'section '.$toc['id']->id;
            else
            $title = $toc['title']->Convert($c);
            $l->addLink($toc['id']->id,$this->_path,basename($this->_path),$toc['id']->package, $toc['id']->subpackage, strip_tags($title));
            $newtoc[$i]['link'] = $c->returnSee($l);
            $newtoc[$i]['id'] = $c->getTutorialId($toc['id']->package, $toc['id']->subpackage, basename($this->_path), $toc['id']->id);
            $newtoc[$i]['title'] = $title;
        }
        return $c->formatTutorialTOC($newtoc);
    }
}
?>