<?php
/**
 * @package phpDocumentor
 * @subpackage Links
 */

/**
 * linking classes
 * @package phpDocumentor
 * @subpackage Links
 */
class abstractLink
{
    var $path;
    /**
     * phpdoc alias _phpdoc_inc for phpdoc.inc
     */
    var $fileAlias = '';
    /**
     * element type linked to.
     * can only be a documentable element
     */
    var $type = '';
    var $name = '';
    var $package = '';
    var $subpackage = '';

    function addLink($path,$fileAlias,$pagename,$package,$subpackage)
    {
        $this->path = $path;
        $this->fileAlias = $fileAlias;
        $this->name = $pagename;
        $this->package = $package;
        $this->subpackage = $subpackage;
    }
}

/**
 * procedural page link
 * @package phpDocumentor
 * @subpackage Links
 */
class pageLink extends abstractLink
{
    var $type = 'page';
    
}

/**
 * function link
 * @package phpDocumentor
 * @subpackage Links
 */
class functionLink extends abstractLink
{
    var $type = 'function';
}

/**
 * define link
 * @package phpDocumentor
 * @subpackage Links
 */
class defineLink extends abstractLink
{
    var $type = 'define';
}

/**
 * global variable link
 * @package phpDocumentor
 * @subpackage Links
 */
class globalLink extends abstractLink
{
    var $type = 'global';
}

/**
 * class link
 * @package phpDocumentor
 * @subpackage Links
 */
class classLink extends abstractLink
{
    var $type = 'class';
}

/**
 * method link
 * @package phpDocumentor
 * @subpackage Links
 */
class methodLink extends abstractLink
{
    var $type = 'method';
    var $class = '';
    
    function addLink($class, $path ,$fileAlias,$name,$package,$subpackage)
    {
        $this->class = $class;
        abstractLink::addLink($path, $fileAlias,$name,$package,$subpackage);
    }
}

/**
 * class variable link
 * @package phpDocumentor
 * @subpackage Links
 */
class varLink extends abstractLink
{
    var $type = 'var';
    var $class = '';

    function addLink($class,$path,$fileAlias,$name,$package,$subpackage)
    {
        $this->class = $class;
        abstractLink::addLink($path,$fileAlias,$name,$package,$subpackage);
    }
}

/**
 * tutorial link
 * @package phpDocumentor
 * @subpackage Links
 */
class tutorialLink extends abstractLink
{
    var $type = 'tutorial';
    var $section = '';
    var $title = false;
    
    function addLink($section,$path,$name,$package,$subpackage,$title = false)
    {
        $this->section = $section;
        $this->title = $title;
        parent::addLink($path,'',$name,$package,$subpackage);
    }
}
?>